import ArrayColumn, { IArrayColumnDesc, IArrayDesc, spliceChanged } from './ArrayColumn';
import Column, { widthChanged, labelChanged, metaDataChanged, dirty, dirtyHeader, dirtyValues, rendererTypeChanged, groupRendererChanged, summaryRendererChanged, visibilityChanged } from './Column';
import ValueColumn, { dataLoaded } from './ValueColumn';
import { IDataRow } from './interfaces';
import { EAdvancedSortMethod, INumberFilter, INumbersColumn } from './INumberColumn';
import { IMapAbleDesc, IMappingFunction } from './MappingFunction';
import { colorMappingChanged } from './NumberColumn';
import { IAdvancedBoxPlotData } from '../internal/math';
import { IColorMappingFunction } from './ColorMappingFunction';
export interface INumbersDesc extends IArrayDesc, IMapAbleDesc {
    readonly sort?: EAdvancedSortMethod;
}
export declare type INumbersColumnDesc = INumbersDesc & IArrayColumnDesc<number>;
export declare function mappingChanged(previous: IMappingFunction, current: IMappingFunction): void;
export declare function sortMethodChanged(previous: EAdvancedSortMethod, current: EAdvancedSortMethod): void;
export default class NumbersColumn extends ArrayColumn<number> implements INumbersColumn {
    static readonly EVENT_MAPPING_CHANGED: string;
    static readonly EVENT_COLOR_MAPPING_CHANGED: string;
    static readonly EVENT_SORTMETHOD_CHANGED: string;
    static readonly CENTER: number;
    private sort;
    private mapping;
    private original;
    private colorMapping;
    private currentFilter;
    constructor(id: string, desc: Readonly<INumbersColumnDesc>);
    compare(a: IDataRow, b: IDataRow): number;
    getRawNumbers(row: IDataRow): number[];
    getBoxPlotData(row: IDataRow): IAdvancedBoxPlotData | null;
    getRange(): [string, string];
    getRawBoxPlotData(row: IDataRow): IAdvancedBoxPlotData | null;
    getNumbers(row: IDataRow): number[];
    getNumber(row: IDataRow): number;
    getRawNumber(row: IDataRow): number;
    getValue(row: IDataRow): number[];
    getRawValue(row: IDataRow): number[];
    getLabels(row: IDataRow): string[];
    getSortMethod(): EAdvancedSortMethod;
    setSortMethod(sort: EAdvancedSortMethod): void;
    dump(toDescRef: (desc: any) => any): any;
    restore(dump: any, factory: (dump: any) => Column | null): void;
    protected createEventList(): string[];
    on(type: typeof NumbersColumn.EVENT_COLOR_MAPPING_CHANGED, listener: typeof colorMappingChanged | null): this;
    on(type: typeof NumbersColumn.EVENT_MAPPING_CHANGED, listener: typeof mappingChanged | null): this;
    on(type: typeof NumbersColumn.EVENT_SORTMETHOD_CHANGED, listener: typeof sortMethodChanged | null): this;
    on(type: typeof ArrayColumn.EVENT_SPLICE_CHANGED, listener: typeof spliceChanged | null): this;
    on(type: typeof ValueColumn.EVENT_DATA_LOADED, listener: typeof dataLoaded | null): this;
    on(type: typeof Column.EVENT_WIDTH_CHANGED, listener: typeof widthChanged | null): this;
    on(type: typeof Column.EVENT_LABEL_CHANGED, listener: typeof labelChanged | null): this;
    on(type: typeof Column.EVENT_METADATA_CHANGED, listener: typeof metaDataChanged | null): this;
    on(type: typeof Column.EVENT_DIRTY, listener: typeof dirty | null): this;
    on(type: typeof Column.EVENT_DIRTY_HEADER, listener: typeof dirtyHeader | null): this;
    on(type: typeof Column.EVENT_DIRTY_VALUES, listener: typeof dirtyValues | null): this;
    on(type: typeof Column.EVENT_RENDERER_TYPE_CHANGED, listener: typeof rendererTypeChanged | null): this;
    on(type: typeof Column.EVENT_GROUP_RENDERER_TYPE_CHANGED, listener: typeof groupRendererChanged | null): this;
    on(type: typeof Column.EVENT_SUMMARY_RENDERER_TYPE_CHANGED, listener: typeof summaryRendererChanged | null): this;
    on(type: typeof Column.EVENT_VISIBILITY_CHANGED, listener: typeof visibilityChanged | null): this;
    getOriginalMapping(): IMappingFunction;
    getMapping(): IMappingFunction;
    setMapping(mapping: IMappingFunction): void;
    getColor(row: IDataRow): any;
    getColorMapping(): IColorMappingFunction;
    setColorMapping(mapping: IColorMappingFunction): void;
    isFiltered(): any;
    getFilter(): INumberFilter;
    setFilter(value?: INumberFilter): void;
    filter(row: IDataRow): any;
}
