import Column, { widthChanged, labelChanged, metaDataChanged, dirty, dirtyHeader, dirtyValues, rendererTypeChanged, groupRendererChanged, summaryRendererChanged, visibilityChanged } from './Column';
import { IDataRow } from './interfaces';
import ValueColumn, { IValueColumnDesc, dataLoaded } from './ValueColumn';
export declare enum EAlignment {
    left = "left",
    center = "center",
    right = "right"
}
export interface IStringDesc {
    alignment?: EAlignment;
    escape?: boolean;
    pattern?: string;
    patternTemplates?: string[];
}
export declare type IStringColumnDesc = IStringDesc & IValueColumnDesc<string>;
export declare function patternChanged(previous: string, current: string): void;
export declare function filterChanged(previous: string | RegExp | null, current: string | RegExp | null): void;
export default class StringColumn extends ValueColumn<string> {
    static readonly EVENT_PATTERN_CHANGED: string;
    static readonly EVENT_FILTER_CHANGED: string;
    static readonly FILTER_MISSING: string;
    private currentFilter;
    readonly alignment: EAlignment;
    readonly escape: boolean;
    private pattern;
    private patternFunction;
    readonly patternTemplates: string[];
    constructor(id: string, desc: Readonly<IStringColumnDesc>);
    setPattern(pattern: string): void;
    getPattern(): string;
    protected createEventList(): string[];
    on(type: typeof StringColumn.EVENT_PATTERN_CHANGED, listener: typeof patternChanged | null): this;
    on(type: typeof StringColumn.EVENT_FILTER_CHANGED, listener: typeof filterChanged | null): this;
    on(type: typeof ValueColumn.EVENT_DATA_LOADED, listener: typeof dataLoaded | null): this;
    on(type: typeof Column.EVENT_WIDTH_CHANGED, listener: typeof widthChanged | null): this;
    on(type: typeof Column.EVENT_LABEL_CHANGED, listener: typeof labelChanged | null): this;
    on(type: typeof Column.EVENT_METADATA_CHANGED, listener: typeof metaDataChanged | null): this;
    on(type: typeof Column.EVENT_DIRTY, listener: typeof dirty | null): this;
    on(type: typeof Column.EVENT_DIRTY_HEADER, listener: typeof dirtyHeader | null): this;
    on(type: typeof Column.EVENT_DIRTY_VALUES, listener: typeof dirtyValues | null): this;
    on(type: typeof Column.EVENT_RENDERER_TYPE_CHANGED, listener: typeof rendererTypeChanged | null): this;
    on(type: typeof Column.EVENT_GROUP_RENDERER_TYPE_CHANGED, listener: typeof groupRendererChanged | null): this;
    on(type: typeof Column.EVENT_SUMMARY_RENDERER_TYPE_CHANGED, listener: typeof summaryRendererChanged | null): this;
    on(type: typeof Column.EVENT_VISIBILITY_CHANGED, listener: typeof visibilityChanged | null): this;
    getValue(row: IDataRow): any;
    getLabel(row: IDataRow): any;
    dump(toDescRef: (desc: any) => any): any;
    restore(dump: any, factory: (dump: any) => Column | null): void;
    isFiltered(): boolean;
    filter(row: IDataRow): boolean;
    getFilter(): string | RegExp | null;
    setFilter(filter: string | RegExp | null): void;
    compare(a: IDataRow, b: IDataRow): any;
}
