import StringColumn from '../model/StringColumn';
import { ERenderMode } from './interfaces';
import { renderMissingDOM } from './missing';
import { noop, noRenderer, setText } from './utils';
var LinkCellRenderer = (function () {
    function LinkCellRenderer() {
        this.title = 'Link';
    }
    LinkCellRenderer.prototype.canRender = function (col, mode) {
        return col instanceof StringColumn && mode !== ERenderMode.SUMMARY;
    };
    LinkCellRenderer.prototype.create = function (col) {
        var align = col.alignment || 'left';
        return {
            template: "<a" + (align !== 'left' ? " class=\"lu-" + align + "\"" : '') + " target=\"_blank\" href=\"\"></a>",
            update: function (n, d) {
                if (renderMissingDOM(n, col, d)) {
                    return;
                }
                n.href = col.getValue(d);
                if (col.escape) {
                    setText(n, col.getLabel(d));
                }
                else {
                    n.innerHTML = col.getLabel(d);
                }
            },
            render: noop
        };
    };
    LinkCellRenderer.exampleText = function (col, rows) {
        var numExampleRows = 5;
        var examples = [];
        for (var _i = 0, rows_1 = rows; _i < rows_1.length; _i++) {
            var row = rows_1[_i];
            if (col.isMissing(row)) {
                continue;
            }
            examples.push("<a target=\"_blank\" href=\"" + col.getValue(row) + "\">" + col.getLabel(row) + "</a>");
            if (examples.length >= numExampleRows) {
                break;
            }
        }
        return "" + examples.join(', ') + (examples.length < rows.length ? ', &hellip;' : '');
    };
    LinkCellRenderer.prototype.createGroup = function (col) {
        return {
            template: "<div> </div>",
            update: function (n, _group, rows) {
                n.innerHTML = "" + LinkCellRenderer.exampleText(col, rows);
            }
        };
    };
    LinkCellRenderer.prototype.createSummary = function () {
        return noRenderer;
    };
    return LinkCellRenderer;
}());
export default LinkCellRenderer;
//# sourceMappingURL=LinkCellRenderer.js.map